part of jaguar.generator.parser.route;

class ParsedExceptionHandler {
  final ElementAnnotation _handler;

  final DartType _exception;

  final MethodElement method;

  String get handlerName => _handler.computeConstantValue().type.name;

  String get exceptionName => _exception.name;

  ParsedExceptionHandler(this._handler, this._exception, this.method) {}

  String get instantiationString => _handler.toSource().substring(1);

  static ParsedExceptionHandler detectException(ElementAnnotation element) {
    if (element.constantValue.type.element is! ClassElement) {
      return null;
    }

    final ClassElement clazz = element.constantValue.type.element;

    final List<InterfaceType> interfaces = clazz.allSupertypes;

    InterfaceType interface;

    for(InterfaceType i in interfaces) {
      if(!isExceptionHandler.isExactlyType(i)) continue;
      interface = i;
      break;
    }

    if (interface is! InterfaceType) return null;

    if (interface.typeArguments.length == 0) {
      throw new GeneratorException(
          '', 0, "ExceptionHandler must specify exception type!");
    }

    MethodElement method = clazz.methods.firstWhere(
        (MethodElement method) => method.name == 'onRouteException',
        orElse: () => null);

    if (method == null) {
      throw new GeneratorException(
          '', 0, "ExceptionHandler must have onRouteException method!");
    }

    return new ParsedExceptionHandler(
        element, interface.typeArguments[0], method);
  }

  static List<ParsedExceptionHandler> detectAllExceptions(Element element) {
    return element.metadata
        .map(detectException)
        .where((value) => value is ParsedExceptionHandler)
        .toList();
  }
}
